<?php

namespace Drupal\Tests\media_entity_link\Functional;

use Drupal\media\MediaInterface;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;
use Drupal\views\Tests\ViewTestData;
use PHPUnit\Framework\Attributes\Group;

/**
 * Tests the LinkMediaLibraryAddForm in a browser.
 */
#[Group('media_entity_link')]
class LinkMediaLibraryAddFormFunctionalTest extends BrowserTestBase {

  use MediaTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'file',
    'image',
    'media',
    'media_library',
    'link',
    'path_alias',
    'media_entity_link',
    'node',
    'block',
    'views',
  ];

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function setUp(): void {
    parent::setUp();

    // Create test views to initialize the views.executable service.
    ViewTestData::createTestViews(
      class: static::class,
      modules: ['views_test_config'],
    );

    // Create a content type with a media field.
    $this->createContentType(values: ['type' => 'page']);
    $this->drupalCreateNode([
      'type' => 'page',
      'title' => 'Test page',
    ]);

    // Place the local actions block, which contains the "Add media" link.
    $this->drupalPlaceBlock(plugin_id: 'local_actions_block');

    // Create a user with the necessary permissions.
    $user = $this->drupalCreateUser([
      'access content',
      'access media overview',
      'create media',
      'view media',
      'administer media',
    ]);
    if ($user) {
      $this->drupalLogin($user);
    }
  }

  /**
   * Tests adding a link media entity through the media library UI.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  public function testAddLinkMediaEntity(): void {
    // Create a media entity programmatically.
    $this->createMedia(
      name: 'Test Link',
      bundle: 'link',
      url: 'http://example.com',
    );

    // Go to the media overview page and verify the media entity exists.
    $this->drupalGet(path: 'admin/content/media');
    $this->assertSession()->pageTextContains(text: 'Test Link');
  }

  /**
   * Creates a media entity.
   *
   * @param string $name
   *   The name of the media entity.
   * @param string $bundle
   *   The bundle of the media entity.
   * @param string $url
   *   The URL for the link.
   *
   * @return \Drupal\media\MediaInterface
   *   The created media entity.
   */
  protected function createMedia(
    string $name,
    string $bundle,
    string $url,
  ): MediaInterface {
    $media = $this->container->get('entity_type.manager')
      ->getStorage('media')
      ->create([
        'bundle' => $bundle,
        'name' => $name,
        'field_media_entity_link' => [
          'uri' => $url,
        ],
        'status' => 1,
      ]);
    $media->save();
    return $media;
  }

  /**
   * Tests adding an internal link media entity.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  public function testAddInternalLinkMediaEntity(): void {
    // Create a media entity programmatically with an internal path.
    $this->createMedia(
      name: 'Internal Link',
      bundle: 'link',
      url: 'internal:/node/1',
    );

    // Go to the media overview page and verify the media entity exists.
    $this->drupalGet(path: 'admin/content/media');
    $this->assertSession()->pageTextContains(text: 'Internal Link');
  }

  /**
   * Tests validation errors for invalid URLs.
   */
  public function testUrlValidation(): void {
    // Test creating a media entity with an empty URL.
    try {
      $this->createMedia(name: 'Empty URL', bundle: 'link', url: '');
      $this->fail('Expected an exception when creating a media entity with an empty URL.');
    }
    catch (\Exception $e) {
      // Verify that an exception was thrown; the exact message may vary.
      $this->addToAssertionCount(1);
    }

    // Test creating a media entity with an invalid URL.
    try {
      $this->createMedia(
        name: 'Invalid URL',
        bundle: 'link',
        url: 'not-a-valid-url',
      );
      $this->fail('Expected an exception when creating a media entity with an invalid URL.');
    }
    catch (\Exception $e) {
      // Verify that an exception was thrown, the exact message may vary.
      $this->addToAssertionCount(1);
    }
  }

  /**
   * Tests the form display for different link type configurations.
   */
  public function testLinkTypeFormDisplay(): void {
    // Create a media type that only allows internal links.
    $internal_only_type = $this->createMediaType(
      source_plugin_id: 'link',
      values: [
        'id' => 'internal_only',
        'label' => 'Internal Only',
      ],
    );
    $source_field = $internal_only_type->getSource()
      ->getSourceFieldDefinition(type: $internal_only_type);
    // LINK_INTERNAL only.
    if (
      $source_field
      && method_exists($source_field, 'setSetting')
      && method_exists($source_field, 'save')
    ) {
      $source_field->setSetting('link_type', 1);
      $source_field->save();
    }

    // Create a media type that only allows external links.
    $external_only_type = $this->createMediaType(
      source_plugin_id: 'link',
      values: [
        'id' => 'external_only',
        'label' => 'External Only',
      ],
    );
    $source_field = $external_only_type->getSource()
      ->getSourceFieldDefinition(type: $external_only_type);
    // LINK_EXTERNAL only.
    if (
      $source_field
      && method_exists($source_field, 'setSetting')
      && method_exists($source_field, 'save')
    ) {
      $source_field->setSetting('link_type', 16);
      $source_field->save();
    }

    // Test creating a media entity with an internal link for the internal-only
    // type.
    $internal_media = $this->container->get('entity_type.manager')
      ->getStorage('media')
      ->create([
        'bundle' => 'internal_only',
        'name' => 'Internal Only Link',
        'field_media_entity_link' => [
          'uri' => 'internal:/node/1',
        ],
        'status' => 1,
      ]);
    $internal_media->save();

    // Test creating a media entity with an external link for the external-only
    // type.
    $external_media = $this->container->get('entity_type.manager')
      ->getStorage('media')
      ->create([
        'bundle' => 'external_only',
        'name' => 'External Only Link',
        'field_media_entity_link' => [
          'uri' => 'http://example.com',
        ],
        'status' => 1,
      ]);
    $external_media->save();

    // Test creating a media entity with an external link for the internal-only
    // type (should fail).
    try {
      $this->container->get('entity_type.manager')
        ->getStorage('media')
        ->create([
          'bundle' => 'internal_only',
          'name' => 'Invalid External Link',
          'field_media_entity_link' => [
            'uri' => 'http://example.com',
          ],
          'status' => 1,
        ])->save();
      $this->fail('Expected an exception when creating an internal-only media entity with an external URL.');
    }
    catch (\Exception $e) {
      // Verify that an exception was thrown, the exact message may vary.
      $this->addToAssertionCount(1);
    }

    // Test creating a media entity with an internal link for the external-only
    // type (should fail).
    try {
      $this->container->get('entity_type.manager')
        ->getStorage('media')
        ->create([
          'bundle' => 'external_only',
          'name' => 'Invalid Internal Link',
          'field_media_entity_link' => [
            'uri' => 'internal:/node/1',
          ],
          'status' => 1,
        ])->save();
      $this->fail('Expected an exception when creating an external-only media entity with an internal URL.');
    }
    catch (\Exception $e) {
      // Verify that an exception was thrown, the exact message may vary.
      $this->addToAssertionCount(1);
    }
  }

}
