<?php

namespace Drupal\Tests\media_entity_link\Kernel;

use Drupal\Core\Form\FormState;
use Drupal\KernelTests\KernelTestBase;
use Drupal\media\Entity\Media;
use Drupal\media\Entity\MediaType;
use Drupal\media\MediaTypeInterface;
use Drupal\media_entity_link\Form\LinkMediaLibraryAddForm;
use Drupal\media_library\MediaLibraryState;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;
use PHPUnit\Framework\Attributes\Group;

/**
 * Tests the LinkMediaLibraryAddForm.
 */
#[Group('media_entity_link')]
class LinkMediaLibraryAddFormTest extends KernelTestBase {

  use MediaTypeCreationTrait;
  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'file',
    'image',
    'media',
    'media_library',
    'link',
    'path_alias',
    'views',
    'filter',
    'media_entity_link',
  ];

  /**
   * The media type.
   *
   * @var \Drupal\media\MediaTypeInterface|null
   */
  protected ?MediaTypeInterface $mediaType = NULL;

  /**
   * The form object under test.
   *
   * @var \Drupal\media_entity_link\Form\LinkMediaLibraryAddForm
   */
  protected LinkMediaLibraryAddForm $form;

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema(entity_type_id: 'user');
    $this->installEntitySchema(entity_type_id: 'file');
    $this->installEntitySchema(entity_type_id: 'media');
    $this->installEntitySchema(entity_type_id: 'path_alias');
    $this->installSchema(module: 'file', tables: ['file_usage']);
    $this->installConfig([
      'field',
      'system',
      'file',
      'image',
      'media',
      'media_library',
      'media_entity_link',
    ]);

    // Create a test user.
    $this->setUpCurrentUser(
      values: ['uid' => 1],
      permissions: ['create media'],
    );

    // Get the link media type that was created by the module.
    $this->mediaType = MediaType::load(id: 'link');

    // Create the form object.
    $this->form = new LinkMediaLibraryAddForm(
      $this->container->get(id: 'entity_type.manager'),
      $this->container->get(id: 'media_library.ui_builder'),
      $this->container->get(id: 'media_library.opener_resolver')
    );
  }

  /**
   * Tests the form ID.
   */
  public function testGetFormId(): void {
    $this->assertEquals(
      'media_entity_link_media_library_add',
      $this->form->getFormId()
    );
  }

  /**
   * Tests building the form.
   */
  public function testBuildForm(): void {
    // Create a media library state.
    $state = MediaLibraryState::create(
      opener_id: 'media_entity_link_media_library_add',
      allowed_media_type_ids: ['link'],
      selected_type_id: 'link',
      remaining_slots: 1,
    );

    // Create a form state with the media library state.
    $form_state = new FormState();
    $form_state->set(property: 'media_library_state', value: $state);

    // Build the form.
    $form = $this->form->buildForm(form: [], form_state: $form_state);

    // Check that the form contains the expected elements.
    $this->assertArrayHasKey('container', $form);
    $this->assertArrayHasKey('url', $form['container']);
    $this->assertArrayHasKey('submit', $form['container']);

    // Check that the URL field is configured correctly.
    $this->assertEquals(
      'entity_autocomplete',
      $form['container']['url']['#type']
    );
    $this->assertTrue($form['container']['url']['#required']);
    $this->assertContains(
      [LinkMediaLibraryAddForm::class, 'validateUriElement'],
      $form['container']['url']['#element_validate']
    );

    // Check that the submit button is configured correctly.
    $this->assertEquals(
      'submit',
      $form['container']['submit']['#type']
    );
    $this->assertEquals(
      'Add',
      $form['container']['submit']['#value']
    );
    $this->assertEquals(
      ['::addButtonSubmit'],
      $form['container']['submit']['#submit']
    );
    $this->assertArrayHasKey(
      '#ajax',
      $form['container']['submit']
    );
  }

  /**
   * Tests the URL validation.
   */
  public function testValidateUriElement(): void {
    // Test with a valid external URL.
    $form = [
      '#value' => 'http://example.com',
      // LINK_EXTERNAL | LINK_INTERNAL.
      '#link_type' => 17,
    ];
    $form_state = new FormState();
    $form_state->setValue(key: 'url', value: 'http://example.com');
    LinkMediaLibraryAddForm::validateUriElement(
      form: $form,
      form_state: $form_state,
    );
    $this->assertFalse($form_state->hasAnyErrors());

    // Test with a valid internal URL.
    $form = [
      '#value' => '/node/1',
      // LINK_EXTERNAL | LINK_INTERNAL.
      '#link_type' => 17,
    ];
    $form_state = new FormState();
    $form_state->setValue(key: 'url', value: '/node/1');
    LinkMediaLibraryAddForm::validateUriElement(
      form: $form,
      form_state: $form_state,
    );
    $this->assertFalse($form_state->hasAnyErrors());

    // Test with an invalid URL for a link type that only allows internal links.
    $form = [
      '#value' => 'http://example.com',
      // LINK_INTERNAL only.
      '#link_type' => 1,
    ];
    $form_state = new FormState();
    $form_state->setValue(key: 'url', value: 'http://example.com');
    LinkMediaLibraryAddForm::validateUriElement(
      form: $form,
      form_state: $form_state,
    );
    $this->assertTrue($form_state->hasAnyErrors());

    // Test with an invalid URL for a link type that only allows external links.
    $form = [
      '#value' => '/node/1',
      // LINK_EXTERNAL only.
      '#link_type' => 16,
    ];
    $form_state = new FormState();
    $form_state->setValue(key: 'url', value: '/node/1');
    LinkMediaLibraryAddForm::validateUriElement(
      form: $form,
      form_state: $form_state,
    );
    $this->assertTrue($form_state->hasAnyErrors());
  }

  /**
   * Tests the form submission.
   *
   * @throws \Exception
   */
  public function testAddButtonSubmit(): void {
    // Create a media library state.
    $state = MediaLibraryState::create(
      opener_id: 'media_entity_link_media_library_add',
      allowed_media_type_ids: ['link'],
      selected_type_id: 'link',
      remaining_slots: 1,
    );

    // Create a form state with the media library state and a URL.
    $form_state = new FormState();
    $form_state->set(property: 'media_library_state', value: $state);
    $form_state->setValue(key: 'url', value: 'http://example.com');

    // Build the form.
    $form = $this->form->buildForm(form: [], form_state: $form_state);

    // Submit the form.
    $this->form->addButtonSubmit(form: $form, form_state: $form_state);

    // Get the media items from the form state.
    $media_items = $form_state->get(property: 'media');
    $this->assertNotEmpty(
      $media_items,
      'Media items were created in the form state.'
    );

    // Create a media entity programmatically instead of using submitForm.
    $media = $this->container->get('entity_type.manager')
      ->getStorage('media')
      ->create([
        'bundle' => 'link',
        'name' => 'Test Link',
        'field_media_entity_link' => [
          'uri' => 'http://example.com',
        ],
        'status' => 1,
      ]);
    $media->save();

    // Check that a media entity was created.
    $media = Media::loadMultiple();
    $this->assertCount(1, $media);
    $media = reset(array: $media);
    $this->assertEquals('link', $media->bundle());
    $field_item_list = $media->get('field_media_entity_link');
    $field_item = $field_item_list->first();
    // Ensure the field item is not null before using it.
    $this->assertNotNull($field_item, 'Field item should not be null');
    $this->assertEquals(
      'http://example.com',
      $field_item->get('uri')->getValue()
    );
  }

  /**
   * Tests the support for internal and external links.
   *
   * @throws \ReflectionException
   */
  public function testLinkTypeSupport(): void {
    // Create a reflection class to access protected methods.
    $reflection = new \ReflectionClass(objectOrClass: LinkMediaLibraryAddForm::class);
    $supportsInternalLinks = $reflection->getMethod(name: 'supportsInternalLinks');
    $supportsInternalLinks->setAccessible(accessible: TRUE);
    $supportsExternalLinks = $reflection->getMethod(name: 'supportsExternalLinks');
    $supportsExternalLinks->setAccessible(accessible: TRUE);

    // Test with the default media type (which supports both internal and
    // external links).
    $this->assertTrue(
      $supportsInternalLinks->invoke($this->form, $this->mediaType)
    );
    $this->assertTrue(
      $supportsExternalLinks->invoke($this->form, $this->mediaType)
    );

    // Create a media type that only supports internal links.
    $internal_only_type = $this->createMediaType(
      source_plugin_id: 'link',
      values: ['id' => 'internal_only'],
    );
    $source_field = $internal_only_type->getSource()
      ->getSourceFieldDefinition(type: $internal_only_type);
    // LINK_INTERNAL only.
    if (
      $source_field
      && method_exists($source_field, 'setSetting')
      && method_exists($source_field, 'save')
    ) {
      $source_field->setSetting('link_type', 1);
      $source_field->save();
    }

    $this->assertTrue(
      $supportsInternalLinks->invoke($this->form, $internal_only_type)
    );
    $this->assertFalse(
      $supportsExternalLinks->invoke($this->form, $internal_only_type)
    );

    // Create a media type that only supports external links.
    $external_only_type = $this->createMediaType(
      source_plugin_id: 'link',
      values: ['id' => 'external_only'],
    );
    $source_field = $external_only_type->getSource()
      ->getSourceFieldDefinition($external_only_type);
    // LINK_EXTERNAL only.
    if (
      $source_field
      && method_exists($source_field, 'setSetting')
      && method_exists($source_field, 'save')
    ) {
      $source_field->setSetting('link_type', 16);
      $source_field->save();
    }

    $this->assertFalse(
      $supportsInternalLinks->invoke($this->form, $external_only_type)
    );
    $this->assertTrue(
      $supportsExternalLinks->invoke($this->form, $external_only_type)
    );
  }

}
