<?php

namespace Drupal\Tests\media_entity_link\Kernel;

use Drupal\media\Entity\Media;
use Drupal\Tests\media\Kernel\MediaKernelTestBase;

/**
 * Tests the link media source plugin.
 *
 * @group media_entity_link
 */
class MediaEntityLinkSourceKernelTest extends MediaKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'media_entity_link',
    'link',
  ];

  /**
   * The test media type.
   *
   * @var \Drupal\media\MediaTypeInterface
   */
  protected $mediaType;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a media type for links using the trait method.
    $this->mediaType = $this->createMediaType('link');
  }

  /**
   * Tests creating a media entity with a link.
   */
  public function testCreateMediaEntity(): void {
    // Get the source field definition and ensure it's not null.
    $source_field_definition = $this->mediaType->getSource()->getSourceFieldDefinition($this->mediaType);
    $this->assertNotNull($source_field_definition, 'Source field definition should not be null');
    $source_field_name = $source_field_definition->getName();

    // Create a media entity with a link.
    $media = Media::create([
      'bundle' => $this->mediaType->id(),
      'name' => 'Test link',
      $source_field_name => [
        'uri' => 'https://www.drupal.org',
        'title' => 'Drupal',
      ],
    ]);
    $media->save();

    // Reload the media entity.
    $media = Media::load($media->id());
    $this->assertNotNull($media, 'Media entity should not be null after loading');

    // Get the field item list and ensure it has the expected values.
    $field_item_list = $media->get($source_field_name);
    // We know the field item list is of the correct type, so just add to the
    // assertion count.
    $this->addToAssertionCount(1);
    $this->assertNotEmpty($field_item_list->getValue(), 'Field item list should have values');

    // Get the first item from the list.
    $field_item = $field_item_list->first();
    // Ensure the field item is not null before using it.
    $this->assertNotNull($field_item, 'Field item should not be null');

    // Assert that the source field value is correct.
    $this->assertEquals('https://www.drupal.org', $field_item->get('uri')->getValue());
    $this->assertEquals('Drupal', $field_item->get('title')->getValue());
  }

  /**
   * Tests the source field configuration.
   */
  public function testSourceFieldConfiguration(): void {
    // Get the source field.
    $source_field = $this->mediaType->getSource()->getSourceFieldDefinition($this->mediaType);

    // Assert that the source field is not null.
    $this->assertNotNull($source_field, 'Source field definition should not be null');

    // Assert that the source field is a link field.
    $this->assertEquals('link', $source_field->getType());

    // Assert that the source field is required.
    $this->assertTrue($source_field->isRequired());

    // Assert that the source field has the correct label.
    $this->assertEquals('Link', $source_field->getLabel());
  }

  /**
   * Tests the view display configuration.
   */
  public function testViewDisplayConfiguration(): void {
    // Get the media type ID and ensure it's a string.
    $media_type_id = (string) $this->mediaType->id();

    // Get the view display.
    $view_display = $this->container->get('entity_display.repository')
      ->getViewDisplay('media', $media_type_id);
    // We know the view display is of the correct type, so just add to the
    // assertion count.
    $this->addToAssertionCount(1);

    // Get the source field definition and ensure it's not null.
    $source_field_definition = $this->mediaType->getSource()->getSourceFieldDefinition($this->mediaType);
    $this->assertNotNull($source_field_definition, 'Source field definition should not be null');
    $source_field_name = $source_field_definition->getName();

    // Prepare the view display for the source field.
    $this->mediaType->getSource()->prepareViewDisplay($this->mediaType, $view_display);
    $view_display->save();

    // Reload the view display to get the updated configuration.
    $view_display = $this->container->get('entity_display.repository')
      ->getViewDisplay('media', $media_type_id);
    // We know the reloaded view display is of the correct type, so just add to
    // assertion count.
    $this->addToAssertionCount(1);

    // Get the component for the source field.
    $component = $view_display->getComponent($source_field_name);
    $this->assertNotNull($component, 'Component for source field should not be null');

    // Assert that the source field has the correct formatter.
    $this->assertEquals('link', $component['type']);

    // Assert that the source field has the correct label display.
    $this->assertEquals('hidden', $component['label']);
  }

}
