<?php

namespace Drupal\Tests\media_entity_link\Unit;

use Drupal\Core\Form\FormState;
use Drupal\media_entity_link\Form\LinkMediaLibraryAddForm;
use Drupal\Tests\UnitTestCase;
use Drupal\link\LinkItemInterface;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;

/**
 * Unit tests for the LinkMediaLibraryAddForm class.
 *
 * @group media_entity_link
 * @coversDefaultClass \Drupal\media_entity_link\Form\LinkMediaLibraryAddForm
 */
#[Group('media_entity_link')]
#[CoversClass(LinkMediaLibraryAddForm::class)]
class LinkMediaLibraryAddFormUnitTest extends UnitTestCase {

  /**
   * Tests the validateUriElement method.
   *
   * @dataProvider providerValidateUriElement
   */
  #[DataProvider('providerValidateUriElement')]
  public function testValidateUriElement(
    string $url,
    int $link_type,
    bool $expected_error,
  ): void {
    $form = [
      '#value' => $url,
      '#link_type' => $link_type,
    ];
    $form_state = new FormState();
    $form_state->setValue('url', $url);

    LinkMediaLibraryAddForm::validateUriElement($form, $form_state);

    $errors = $form_state->getErrors();
    $has_url_error = isset($errors['url']);
    $this->assertEquals($expected_error, $has_url_error);
  }

  /**
   * Data provider for testValidateUriElement.
   *
   * @return array<int, array{0: string, 1: int, 2: bool}>
   *   An array of test cases, each containing a URL, link type, and expected
   *   error flag.
   */
  public static function providerValidateUriElement(): array {
    return [
      // Valid external URL with both link types allowed.
      ['http://example.com', LinkItemInterface::LINK_EXTERNAL | LinkItemInterface::LINK_INTERNAL, FALSE],

      // Valid internal URL with both link types allowed.
      ['/node/1', LinkItemInterface::LINK_EXTERNAL | LinkItemInterface::LINK_INTERNAL, FALSE],

      // Valid <front> URL with both link types allowed.
      ['<front>', LinkItemInterface::LINK_EXTERNAL | LinkItemInterface::LINK_INTERNAL, FALSE],

      // External URL with only internal links allowed (should fail).
      ['http://example.com', LinkItemInterface::LINK_INTERNAL, TRUE],

      // Internal URL with only external links allowed (should fail).
      ['/node/1', LinkItemInterface::LINK_EXTERNAL, TRUE],

      // <front> URL with only external links allowed (should fail).
      ['<front>', LinkItemInterface::LINK_EXTERNAL, TRUE],

      // Generic link type allows any URL.
      ['http://example.com', LinkItemInterface::LINK_GENERIC, FALSE],
      ['<front>', LinkItemInterface::LINK_GENERIC, FALSE],
      ['/node/1', LinkItemInterface::LINK_GENERIC, FALSE],
    ];
  }

  /**
   * Tests the getUserEnteredStringAsUri method.
   *
   * @dataProvider providerGetUserEnteredStringAsUri
   *
   * @throws \ReflectionException
   */
  #[DataProvider('providerGetUserEnteredStringAsUri')]
  public function testGetUserEnteredStringAsUri(
    string $input,
    string $expected_uri,
  ): void {
    $reflection = new \ReflectionClass(LinkMediaLibraryAddForm::class);
    $method = $reflection->getMethod('getUserEnteredStringAsUri');
    $method->setAccessible(TRUE);

    $actual_uri = $method->invoke(NULL, $input);
    $this->assertEquals($expected_uri, $actual_uri);
  }

  /**
   * Data provider for testGetUserEnteredStringAsUri.
   *
   * @return array<int, array{0: string, 1: string}>
   *   An array of test cases, each containing an input string and expected URI.
   */
  public static function providerGetUserEnteredStringAsUri(): array {
    return [
      // External URLs should remain unchanged.
      ['http://example.com', 'http://example.com'],
      ['https://example.com', 'https://example.com'],

      // Internal paths should be prefixed with 'internal:'.
      ['/node/1', 'internal:/node/1'],
      ['node/1', 'internal:node/1'],

      // <front> should be converted to 'internal:/'.
      ['<front>', 'internal:/'],
      ['<front>#anchor', 'internal:/#anchor'],

      // <nolink> and <none> should be converted to 'route:' URIs.
      ['<nolink>', 'route:<nolink>'],
      ['<none>', 'route:<none>'],

      // Entity autocomplete strings should be converted to 'entity:' URIs.
      // Note: This is mocked since we can't test entity autocomplete in a unit
      // test. The actual extraction is tested in the kernel test.
    ];
  }

  /**
   * Tests the getFormId method.
   *
   * @throws \ReflectionException
   */
  public function testGetFormId(): void {
    $form = $this->getMockBuilder(LinkMediaLibraryAddForm::class)
      ->disableOriginalConstructor()
      ->getMock();

    $reflection = new \ReflectionClass(LinkMediaLibraryAddForm::class);
    $method = $reflection->getMethod('getFormId');
    $method->setAccessible(TRUE);

    $this->assertEquals(
      'media_entity_link_media_library_add',
      $method->invoke($form)
    );
  }

}
